<?php defined('SYSPATH') OR die('No direct access allowed.');
/**
 * Geomap controller
 *
 *  op5, and the op5 logo are trademarks, servicemarks, registered servicemarks
 *  or registered trademarks of op5 AB.
 *  All other trademarks, servicemarks, registered trademarks, and registered
 *  servicemarks mentioned herein may be the property of their respective owner(s).
 *  The information contained herein is provided AS IS with NO WARRANTY OF ANY
 *  KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY, AND FITNESS FOR A
 *  PARTICULAR PURPOSE.
 */
class Geomap_Controller extends Authenticated_Controller
{
	private static function fix_data($data)
	{
		if (isset($data['object_type'])) {
			$data['object'] = array('type' => $data['object_type']);
			unset($data['object_type']);

			if ($data['object']['type'] == 'service') {
				if (isset($data['object_host'])) {
					$data['object']['host'] = $data['object_host'];
					unset($data['object_host']);
				}

				if (isset($data['object_description'])) {
					$data['object']['description'] = $data['object_description'];
					unset($data['object_description']);
				}
			}
			else
			{
				if (isset($data['object_name'])) {
					$data['object']['name'] = $data['object_name'];
					unset($data['object_name']);
				}
			}
		}
		return $data;
	}

	public function __construct()
	{
		Session::instance();
		if (Router::$method == 'get_raw_view' && isset($_SESSION['external_widget_username'])) {
			$conf = array(
				'enable_auto_login' => true,
				'default_auth' => 'Session',
				'session_key' => false,
			);

			$driver = array(
				'Session' => array(
					'driver' => 'Session',
					'username_session_key' => 'external_widget_username',
					'groups_session_key' => 'external_widget_groups'
				)
			);
			Auth::factory($conf, $driver);
		}elseif (Router::$method == 'get_raw_view' && preg_match("/external_widget/", $_SERVER['HTTP_REFERER'])) {
			//User not logged in and request to load external_widget
			$conf = Kohana::config('external_widget');
			$username = $conf["username"];
			$groups = $conf["groups"];

			$this->op5auth = op5auth::instance(array(
								'enable_auto_login' => true 
							));
			$this->op5auth->get_user();
			$this->op5auth->write_close();
			$user = new User_Model(array(
						'username' => $username,
						'groups' => $groups 
					));
			$this->op5auth->force_user($user);
		}  

		parent::__construct();
		$auth = op5auth::instance();
		if (!$auth->authorized_for('host_view_all')) {
			$this->template->content = $this->add_view('unauthorized');
			$this->template->content->error_message = _('It appears as though you do not have permission to view information you requested...');
			$this->template->content->error_description = _('If you believe this is an error, check the HTTP server authentication requirements for accessing this page
			and check the authorization options in your CGI configuration file.');
			return;
		}
	}

	public function index($is_widget = false)
	{
		$this->template->content = $this->add_view('geomap/index');
		$this->template->js[] = 'modules/geomap/views/js/jquery.form.js';
		$this->template->js[] = 'modules/geomap/views/js/geomap.js';
		if($is_widget) {
			$mangled_js = array();
			foreach($this->template->js as $orig_js) {
				$delim = (strpos($orig_js, '?') === false) ? '?' : '&';
				$mangled_js[] = $orig_js . $delim . 'request_context=external_widget';
			}
			$data['js'] = $mangled_js;
			$this->template->js = $data['js'];
		}
		$this->template->css[] = 'modules/geomap/media/leaflet/leaflet.css';
		$this->template->css[] = 'modules/geomap/views/css/geomap.css';

		$geomap = new Geomap_Model();
		$settings = $geomap->get_settings();
		$edit_mode = $this->input->get('edit');

		switch ($settings['provider'])
		{
			case 'google':
				$this->template->js[] = 'https://maps.google.com/maps/api/js';
				$this->template->js[] = 'modules/geomap/views/js/geomap.googlemaps.js';
				break;

			case 'osm':
				$this->template->js[] = 'modules/geomap/media/leaflet/leaflet.js';
				$this->template->js[] = 'modules/geomap/views/js/geomap.leaflet.js';
				break;

			case 'local':
				$this->template->js[] = 'modules/geomap/media/leaflet/leaflet.js';
				$this->template->js[] = 'modules/geomap/views/js/geomap.local.js';
				break;
		}
		if($is_widget) {
			$this->template->content->is_widget = true;
		} else {
			$title = 'Geomap';
			if($edit_mode) {
				$title .= ' (edit mode)';
			}
			$toolbar = new Toolbar_Controller($title);
			$menu = new Menu_Model();
			$menu->set("Geomap options.Edit mode",
				LinkProvider::factory()
					->get_url(
						'geomap',
						'index',
						array('edit' => 1)
					)
			);
			$menu->set("Geomap options.View mode",
				LinkProvider::factory()
					->get_url(
						'geomap',
						'index'
					)
			);
			$menu->set("Geomap options.Settings",
				LinkProvider::factory()
					->get_url(
						'geomap',
						'index'
					), null, false, array(
				'id' => 'geomap_settings'
			));
			$toolbar->menu($menu);
			$this->template->toolbar = $toolbar;
		}
		$this->template->content->settings = $settings;
		$this->template->content->edit_mode = $edit_mode;

		$this->template->content->title = _('Geomap');
		$this->template->title = _('Monitoring » Geomap');
		$this->template->disable_refresh = true;

		$current_skin = config::get('config.current_skin', '*', true);
		if($current_skin) {
			$current_skin = rtrim($current_skin, '/').'/';
		} else {
			$current_skin = 'default/';
		}
		$this->template->current_skin = $current_skin;
	}

	public function get_raw_view() {
		// we cannot call `new View()` here because js-hooks
		// have already bound bundled js to the existing view
		$this->template->set_filename('chromeless');
		$this->index(true);
	}

	public function locations()
	{
		$geomap = new Geomap_Model();
		echo json_encode(array('items' => $geomap->get_locations(), 'updated' => date('Y-m-d H:i:s')));
		$this->auto_render = false;
	}

	public function links()
	{
		$geomap = new Geomap_Model();
		print json_encode($geomap->get_links());
		$this->auto_render = false;
	}

	public function viewpoints()
	{
		$geomap = new Geomap_Model();
		echo json_encode($geomap->get_viewpoints());
		$this->auto_render = false;
	}

	public function add_location()
	{
		$geomap = new Geomap_Model();
		echo $geomap->add_location(self::fix_data($this->input->post()));
		$this->auto_render = false;
	}

	public function add_link()
	{
		$geomap = new Geomap_Model();
		echo $geomap->add_link($this->input->post());
		$this->auto_render = false;
	}

	public function edit_location()
	{
		$geomap = new Geomap_Model();
		$geomap->edit_location(self::fix_data($this->input->post()));
		$this->auto_render = false;
	}

	public function edit_link()
	{
		$geomap = new Geomap_Model();
		$geomap->edit_link(self::fix_data($this->input->post()));
		$this->auto_render = false;
	}

	public function remove_location()
	{
		$geomap = new Geomap_Model();
		$geomap->remove_location($this->input->post());
		$this->auto_render = false;
	}

	public function remove_link()
	{
		$geomap = new Geomap_Model();
		$geomap->remove_link($this->input->post());
		$this->auto_render = false;
	}

	public function hosts()
	{
		$hosts = HostPool_Model::all();
		$data = array();
		foreach($hosts->it(array('name')) as $row) {
			$data[] = $row->get_name();
		}
		json::ok($data);
	}

	public function savesettings()
	{
		$this->auto_render = false;
		$data = $this->input->get();
		$model = new Geomap_Model();
		print json_encode(array( 'success' => $model->save_settings($data) ));
	}

	public function services()
	{
		$services = ServicePool_Model::all();
		$data = array();
		foreach($services->it(array('host.name', 'description')) as $row) {
			$data[] = $row->get_host()->get_name().";".$row->get_description();
		}
		json::ok($data);
	}

	public function hostgroups()
	{
		$hostgroups = HostGroupPool_Model::all();
		$data = array();
		foreach($hostgroups->it(array('name')) as $row) {
			$data[] = $row->get_name();
		}
		json::ok($data);
	}

	public function servicegroups()
	{
		$servicegroups = ServiceGroupPool_Model::all();
		$data = array();
		foreach($servicegroups->it(array('name')) as $row) {
			$data[] = $row->get_name();
		}
		json::ok($data);
	}
}
