var geomap = (function(){
	var base_url;
	var tile_url;
	var attrib = 'Map data (C) OpenStreetMap contributors';
	var edit_mode;
	var iframe_mode = false;
	var map;
	var markers;
	var lines;
	var new_marker;
	var states = {
		'host': {
			0: 'up',
			1: 'down',
			2: 'unreachable',
			6: 'pending'},
		'service': {
			0: 'ok',
			1: 'warning',
			2: 'critical',
			3: 'unknown',
			6: 'pending'}};
	var icons = {};
	var colors = {
		'host': ['#00FF00', '#FF0000', '#FF9900', '#FF9900',,,'#999999'],
		'service': ['#00FF00', '#FFFF00', '#FF0000', '#FF9900',,,'#999999']};
	var jsonp_callbacks = [];
	var search_markers = [];

	var mapQuestCallback = function(json){
		var ret_val = [];
		for (var i = 0; i < json.length; i++)
			ret_val.push([json[i].lat, json[i].lon]);

		jsonp_callbacks.shift().call(window, ret_val);
	};

	window.mapQuestCallback = mapQuestCallback;

	return {
		type: "leaflet",
		init: function(options){
			base_url = options.base_url;
			edit_mode = options.edit_mode;
			iframe_mode = options.iframe_mode || false;
			tile_url = 'http://{s}.tile.cloudmade.com/' + options.cloudmade_key + '/998/256/{z}/{x}/{y}.png';

			map = new L.Map('map', {
				center: new L.LatLng(options.center[0], options.center[1]),
				zoom: options.zoom,
				minZoom: 2,
				doubleClickZoom: false,
				closePopupOnClick: false
			});
			map.addLayer(new L.TileLayer(tile_url, {attribution: attrib}));
			markers = new L.LayerGroup();
			lines = new L.LayerGroup();

			if (edit_mode)
			{
				map.on('dblclick', function(event){
					if (new_marker)
						map.removeLayer(new_marker);
					new_marker = new L.Marker(event.latlng);
					new_marker.bindPopup('<a href="' + base_url + 'index.php/geomap/index?edit=1" class="add_location" ' +
						+ 'latlng="' + event.latlng.lat + ',' + event.latlng.lng + '">Add location</a>'
						+ '<br><a href="#" onclick="geomap.onCancel(); return false;">Cancel</a>', {closeButton: false});
					map.addLayer(new_marker);
					new_marker.openPopup();
				});
			}

			for (var type in states) {
				icons[type] = {};
				for (var state in states[type]) {
					icons[type][state] = L.Icon.extend({
						options: {
							iconUrl: base_url + 'application/views/icons/16x16/shield-' + states[type][state] + '.png',
							shadowUrl: null,
							iconSize: new L.Point(16, 16),
							iconAnchor: new L.Point(8, 8),
							popupAnchor: new L.Point(0, 0)
						}
					});
				}
			}
		},
		populate: function(locations, links){
			for (var i in locations)
			{
				var location = locations[i];
				var coords = location.point.split(',');
				var base_type = '';
				if (location.object && location.object.type) {
					base_type = location.object.type;
					if (!states[base_type])
						base_type = base_type.slice(0, -5);
				}

				var info = '<strong>' + location.label + '</strong>';
				if (location.description)
					info += '<br>' + location.description;
				if (location.object)
				{
					var object = location.object;
					info += '<br>' + object.type + ': <a'+(iframe_mode ? ' target="_top"' : '') +' href="' + geomap.onDetails(object) + '"><strong>'
						+ (object.type == 'service'	? object.description + '</strong> at <strong>' + object.host : object.name)
						+ '</strong></a><br>state: <strong>' + (states[base_type][object.state] ? states[base_type][object.state] : 'unknown ('+object.state+')') + '</strong>';
				}
				if (edit_mode)
				{
					info += '<br><a href="#" onclick="geomap.onLink(\'' + location.id + '\'); return false;">Link</a>'
						+ '<br><a class="edit_location" href="' + base_url + 'index.php/geomap/index?edit=1" location_id="' + location.id + '">Edit</a>'
						+ '<br><a href="#" class="remove_location" location_id="' + location.id + '">Remove</a>';
				}

				var options = {};
				if(location.object && location.object.state != undefined && icons[base_type][location.object.state]) {
					options.icon = new icons[base_type][location.object.state];
					options.zIndexOffset = location.object.state * 200;
				} else {
					options.icon = new icons['host'][6];
				}
				var marker = new L.Marker(
					new L.LatLng(parseFloat(coords[0]), parseFloat(coords[1])),
					options
				);
				marker.bindPopup(info);
				markers.addLayer(marker);
			}

			map.addLayer(markers);

			for (var i in links)
			{
				var link = links[i];
				var coords1;
				var coords2;
				var base_type = '';
				if (link.object && link.object.type) {
					base_type = link.object.type;
					if (!states[base_type])
						base_type = base_type.slice(0, -5);
				}

				if (locations[link.id1] && locations[link.id2])
				{
					coords1 = locations[link.id1].point.split(',');
					coords2 = locations[link.id2].point.split(',');
				}
				else
					continue;

				var info = '<strong>' + link.description + '</strong>';
				if (link.object)
				{
					var object = link.object;
					info += '<br>' + object.type + ': <a'+(iframe_mode ? ' target="_top"' :'') +' href="' + geomap.onDetails(object) + '"><strong>'
						+ (object.type == 'service'	? object.description + '</strong> at <strong>' + object.host : object.name)
						+ '</strong></a><br>state: <strong>' + (states[base_type][object.state] ? states[base_type][object.state] : 'unknown ('+object.state+')') + '</strong>';
				}
				if (edit_mode)
				{
					info += '<br><a href="#" onclick="geomap.onEditLink(\'' + link.id + '\'); return false;">Edit</a>'
						+ '<br><a href="#" onclick="geomap.onRemoveLink(\'' + link.id + '\'); return false;">Remove</a>';
				}

				var line = new L.Polyline([
					new L.LatLng(parseFloat(coords1[0]), parseFloat(coords1[1])),
					new L.LatLng(parseFloat(coords2[0]), parseFloat(coords2[1]))
				], {
					color: link.object && link.object.state != undefined && colors[base_type][link.object.state] ? colors[base_type][link.object.state] : colors['host'][6],
					opacity: 0.5,
					weight: 5
				});
				line.bindPopup(info);
				lines.addLayer(line);
			}

			map.addLayer(lines);
		},
		clear: function(){
			this.closeNew();

			for (var i = 0; i < search_markers.length; i++)
			{
				map.removeLayer(search_markers[i]);
				delete search_markers[i];
			}
			search_markers = [];

			map.removeLayer(lines);
			lines = new L.LayerGroup();
			map.removeLayer(markers);
			markers = new L.LayerGroup();
		},
		closeNew: function() {
			map.closePopup();
			if (new_marker)
			{
				map.removeLayer(new_marker);
				new_marker = undefined;
			}
		},
		search: function(address, callback){
			jsonp_callbacks.push(callback);
			var s = document.createElement('script');
			s.type = 'text/javascript';
			s.src = 'http://open.mapquestapi.com/nominatim/v1/search?format=json&json_callback=mapQuestCallback&q=' + encodeURIComponent(address);
			document.body.appendChild(s);
		},
		showSearchMarkers: function(list){
			if (!edit_mode)
				return false;

			var mark;
			while (mark = search_markers.shift())
				map.removeLayer(mark);

			for (var i = 0; i < list.length; i++)
			{
				var marker = new L.Marker(new L.LatLng(parseFloat(list[i][0]), parseFloat(list[i][1])));
				map.addLayer(marker);

				(function(indx, mrk){
					mrk.bindPopup('<a href="' + base_url + 'index.php/geomap/index?edit=1" class="add_location" '
						+ 'latlng="' + parseFloat(list[indx][0]) + ',' + parseFloat(list[indx][1]) + '">Add location</a>'
						+ '<br><a href="#" onclick="geomap.onCancel(); return false;">Cancel</a>', {closeButton: false});
				})(i, marker);
				search_markers.push(marker);
			}
		},
		getViewPoint: function(){
			return {
				center: [map.getCenter().lat, map.getCenter().lng],
				zoom: map.getZoom()
			};
		}
	};
})();


